
#ifndef _MAELSTROM_HASHMAP_H_
#define _MAELSTROM_HASHMAP_H_

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include "list.h"

#define unlikely

#define HAVE_BUG_ON

#ifdef HAVE_BUG_ON
#define BUG_ON(condition)	\
	do { if (condition) fprintf(stderr, "BUG_ON\n"); exit(1); } while(0)
#else /* !HAVE_BUG_ON */
#define BUG_ON(condition)	do { if (condition) ; } while (0)
#endif /* HAVE_BUG_ON */

/*
 * Copyright (C) 2008 Tudor Marian (tudorm@cs.cornell.edu)
 *
 * Hash Map implementation (map with a backing hashtable). Not thread-safe.
 */

struct hash_head
{
	void *key;
	struct list_head list;
};

/* chaining for collisions */
struct hash_chain
{
	int len;
	struct list_head head;
};

struct hashmap
{
	struct hash_chain *array;
	int length;
	int bin_limit; /* non-positive means there is no per bin limit */
	unsigned long (*hash)(void *key);
	int (*cmp)(void *key1, void *key2);
	void (*overflow)(struct hash_head *victim);
};

int init_hashmap(struct hashmap *map, int length, int bin_limit,
		void * (*alloc_fn)(unsigned long),
		unsigned long (*hash_fn) (void *),
		int (*cmp_fn)(void *, void *), 
		void (*bin_overflow)(struct hash_head *));

struct hashmap *create_hashmap(int length, int bin_limit,
		void * (*alloc_fn)(unsigned long),
		unsigned long (*hash_fn) (void *),
		int (*cmp_fn)(void *, void *), 
		void (*bin_overflow)(struct hash_head *));

#define create_hashmap_nolimit(length, allocfn, hashfn, cmpfn)		\
		create_hashmap(length, 0, allocfn, hashfn, cmpfn, NULL)

struct hash_head *put_item(struct hashmap *map, struct hash_head *item);
struct hash_head *get_item(struct hashmap *map, void *key);
struct hash_head *remove_item(struct hashmap *map, void *key);
int contains_key(struct hashmap *map, void *key);
void clear(struct hashmap *map, void (*free_item)(struct hash_head *));

static inline void hashmap_release(struct hashmap *map, 
		void (*free_item)(struct hash_head *), 
		void (*free_table)(void *))
{
	if (map->array)
	{
		clear(map, free_item); /* walk the array and clean the bins */
		free_table(map->array); /* free the hashtable array itself */
		map->array = NULL; /* redundant if next instruction is issued */
		memset(map, 0, sizeof(struct hashmap)); /* wipe out content */
	}
}

#endif //_MAELSTROM_HASHMAP_H_


