
/*
 * Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>
 */

#ifndef __NETSLICE_HDR__
#define __NETSLICE_HDR__

#ifndef __KERNEL__
#include <sys/types.h> /* for size_t */
#include <stdint.h> /* for uint32_t, uint16_t */
#endif //!__KERNEL__

#define NETSLICE_PRE_TX_CSUM_INVALID		-1
#define NETSLICE_PRE_TX_CSUM_NONE			0
#define NETSLICE_PRE_TX_CSUM_IP				1
#define NETSLICE_PRE_TX_CSUM_TRANSPORT		2
#define NETSLICE_PRE_TX_CSUM_MAX			3

#define netslice_pre_tx_csum_valid(idx)	\
	((idx >= NETSLICE_PRE_TX_CSUM_NONE) && (idx < NETSLICE_PRE_TX_CSUM_MAX))

#define NETSLICE_PRE_TX_CSUM_SET			_IOW('p', 0x01, int)
#define NETSLICE_PRE_TX_CSUM_GET			_IOR('p', 0x02, int)

struct netslice_rw_vect
{
#define NETSLICE_READV_DATAGRAM_READ		0x1
#define NETSLICE_WRITEV_DATAGRAM_WRITE		0x2
	int flags;
};

#define NETSLICE_READV_WRITEV_SET			_IOW('p', 0x03, struct netslice_rw_vect)
#define NETSLICE_READV_WRITEV_GET			_IOR('p', 0x04, struct netslice_rw_vect)

struct netslice_iov
{
	void *iov_base;
	/* the capacity of the iov_base */
	size_t iov_len;
	/* returned length, value written back upon return from read
	also, the value represents the number of bytes from iov_base that are to be
	written (note that it should be <= iov_len) */
	size_t iov_rlen;
#define NETSLICE_IOV_SKIP_PACKET_MASK		0x1
#define NETSLICE_IOV_CSUM_MASK				0x2 /* at least IP checksum required */
#define NETSLICE_IOV_CSUM_TRANSPORT_MASK	0x4 /* IP & TCP checksum required */
	int flags; /* relay per-packet information (currently only valid 
		if information travels from user-space to kernel) */
};

/* addresses in network endianness for fast bitwise mask operations, 
 ports in host endianness for arithmetic range operations. Mask set to 0x0
 will means that all addresses are valid. */
struct netslice_filter
{
#define NETSLICE_FILTER_IPPROTO_FRAG		0x1
#define NETSLICE_FILTER_IPPROTO_TCP			0x2
#define NETSLICE_FILTER_IPPROTO_UDP			0x4
	int ipproto;
	uint32_t saddr, smask, daddr, dmask;
	int sport_base, sport_cnt, dport_base, dport_cnt;
};

#define NETSLICE_FILTER_SET					_IOW('p', 0x05, struct netslice_filter)
#define NETSLICE_FILTER_GET					_IOR('p', 0x06, struct netslice_filter)

/* currently it supports at most (sizeof(long long) * 8) processors */
struct netslice_cpu_mask
{
	unsigned long long k_peer, u_peer;
};
#define NETSLICE_CPU_MASK_GET				_IOW('p', 0x07, struct netslice_cpu_mask)

#endif //__NETSLICE_HDR__
