
/*
 * Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>
 */

#ifndef __UTIL_HDR__
#define __UTIL_HDR__

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/uts.h>
#include <linux/utsname.h>
#include <linux/version.h>
#include <linux/netfilter_ipv4.h>
#include <linux/kthread.h>
#include <linux/netdevice.h>
#include <linux/inet.h> /* in_aton */
#include <linux/udp.h>
#include <net/ip.h> /* IP_OFFSET */
#include <net/tcp.h> /* tcp_v4_check */
#include <linux/proc_fs.h>
#include <linux/highmem.h>

#include <linux/sched.h>

#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/mm.h>
#ifdef MODVERSIONS
#  include <linux/modversions.h>
#endif
#include <linux/io.h>
#include <linux/kallsyms.h>

//#define NF_HOOK_OUT_SKB

#define is_pow_of_2(x)	( (x) && ! ((x) & ((x)-1)) )
static inline int log2_64(uint64_t x) { return fls64(x-1); }

static inline void transmit_skb(struct sk_buff *skb)
{
#ifdef NF_HOOK_OUT_SKB
	int out_hook;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 24)
	out_hook = NF_IP_LOCAL_OUT;
#else
	out_hook = NF_INET_LOCAL_OUT;
#endif
	NF_HOOK(PF_INET, out_hook, skb, NULL, skb->dst->dev, dst_output);
#else
	dst_output(skb);
#endif /* NF_HOOK_OUT_SKB */
}

static inline struct iphdr *skb_ip_hdr(const struct sk_buff *skb)
{
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 20)
	struct iphdr *iph = skb->nh.iph;
	return iph;
#else
	return ip_hdr(skb);
#endif
}

static inline unsigned char *ip_payload(struct iphdr *iph)
{
	unsigned char *payload = (void *)iph + iph->ihl*4;
	return payload;	
}

/* do not use tcp_hdr since the transport pointer in skb may not be 
 * updated early enough, e.g. if packet is captured via netfilter */
static inline struct tcphdr *ip_tcp_hdr(struct iphdr *iph)
{
	struct tcphdr *tcph = (void *)iph + iph->ihl*4;
	return tcph;	
}

static inline struct tcphdr *skb_tcp_hdr(const struct sk_buff *skb)
{
	return ip_tcp_hdr(skb_ip_hdr(skb));
}

static inline struct udphdr *ip_udp_hdr(struct iphdr *iph)
{
	struct udphdr *udph = (void *)iph + iph->ihl*4;
	return udph;
}

static inline struct udphdr *skb_udp_hdr(const struct sk_buff *skb)
{
	return ip_udp_hdr(skb_ip_hdr(skb));
}

static inline long skb_ip_hdrlen(const struct sk_buff *skb)
{
	struct iphdr *iph = skb_ip_hdr(skb);
	return iph->ihl*4;
}

/* identical to something like PROC_IDE_READ_RETURN */
#define PROC_READ_RETURN(page,start,off,count,eof,len) \
{									\
	len -= off;                     \
	if (len < count) {              \
		*eof = 1;					\
		if (len <= 0)				\
			return 0;       		\
	} else /* truncate */ 			\
		len = count;            	\
	*start = page + off;            \
	return len;                     \
}

#endif //__UTIL_HDR__

